package cli

import (
	"context"
	"fmt"
	"strings"

	"code.fbi.h-da.de/cocsn/gosdn/forks/goarista/gnmi"
	"code.fbi.h-da.de/cocsn/gosdn/nucleus"
	gpb "github.com/openconfig/gnmi/proto/gnmi"
)

// Capabilities sends a gNMI Capabilities request to the specified target
// and prints the supported models to stdout
func Capabilities(a, u, p string) error {
	cfg := gnmi.Config{
		Addr:     a,
		Username: u,
		Password: p,
		Encoding: gpb.Encoding_JSON_IETF,
	}
	opts := &nucleus.GnmiTransportOptions{Config: cfg}
	transport, err := nucleus.NewGnmiTransport(opts)
	if err != nil {
		return err
	}
	resp, err := transport.Capabilities(context.Background())
	if err != nil {
		return err
	}
	modelData := resp.(*gpb.CapabilityResponse).SupportedModels
	b := strings.Builder{}
	for _, elem := range modelData {
		_, err := b.WriteString(elem.Name)
		if err != nil {
			return err
		}
		_, err = b.WriteString("\n")
		if err != nil {
			return err
		}
	}
	fmt.Println(b.String())
	return nil
}
