package cli

import (
	"errors"
	"fmt"
	"io/ioutil"
	"net/http"
	"strings"

	log "github.com/sirupsen/logrus"
	"github.com/spf13/viper"
)

const apiRoot = "?"

var builder *strings.Builder

func init() {
	builder = &strings.Builder{}
}

// HTTPGet sends sends requests from the CLI to the gosdn HTTP API and processes any response data
func HTTPGet(apiEndpoint, f string, args ...string) error {
	for _, p := range args {
		builder.WriteString("&")
		builder.WriteString(p)
	}
	resp, err := http.Get(apiEndpoint + apiRoot + "q=" + f + builder.String())
	if err != nil {
		log.Info(fmt.Sprintf("Err: %s", err))
		return err
	}
	builder.Reset()
	switch resp.StatusCode {
	case http.StatusOK:
		defer resp.Body.Close()
		bytes, err := ioutil.ReadAll(resp.Body)
		if err != nil {
			return err
		}
		//if this is in response to "init" query, we also set PND and SBI to ENV as initialized by the controller
		if f == "init" {
			pnd := string(bytes[9:45])
			sbi := string(bytes[55:91])
			viper.Set("CLI_PND", pnd)
			viper.Set("CLI_SBI", sbi)
			err := viper.WriteConfig()
			if err != nil {
				log.Error(err)
			}
		}
		fmt.Println(string(bytes))

	case http.StatusCreated:
		defer resp.Body.Close()
		bytes, err := ioutil.ReadAll(resp.Body)
		if err != nil {
			return err
		}
		uuid := string(bytes[19:55])
		viper.Set("LAST_DEVICE_UUID", uuid)
		fmt.Println(string(bytes))

	case http.StatusAccepted:
	default:
		log.WithFields(log.Fields{
			"status code": resp.StatusCode,
		}).Error("operation unsuccessful")
		return errors.New(resp.Status)
	}
	return nil
}
