package nucleus

import (
	"code.fbi.h-da.de/cocsn/gosdn/nucleus/errors"
	"github.com/google/uuid"
	"github.com/openconfig/ygot/ygot"
)

// Device represents an Orchestrated Network Device (OND) which is managed by
// nucleus
type Device struct {
	// UUID represents the Devices UUID
	UUID uuid.UUID

	// Device inherits properties of ygot.GoStruct
	ygot.GoStruct

	// SBI is the device's southbound interface implementation
	SBI SouthboundInterface

	// Transport is the device's Transport implementation
	Transport Transport
}

// NewDevice creates a Device
func NewDevice(sbi SouthboundInterface, opts TransportOptions) (*Device, error) {
	var transport Transport
	var err error
	switch o := opts.(type) {
	case *GnmiTransportOptions:
		transport, err = NewGnmiTransport(o)
		if err != nil {
			return nil, err
		}
	default:
		return nil, &errors.ErrInvalidTransportOptions{Opt: o}

	}
	return &Device{
		UUID:      uuid.New(),
		GoStruct:  sbi.Schema().Root,
		SBI:       sbi,
		Transport: transport,
	}, nil
}

// ID returns the UUID of the Device
func (d *Device) ID() uuid.UUID {
	return d.UUID
}
