package nucleus

import (
	"bytes"
	"context"
	"io"

	"github.com/openconfig/ygot/ytypes"
)

// Transport provides an interface for Transport implementations
// like RESTCONF or gnmi
type Transport interface {
	Get(ctx context.Context, params ...string) (interface{}, error)
	Set(ctx context.Context, params ...interface{}) error
	Subscribe(ctx context.Context, params ...string) error
	Type() string
	GetOptions() interface{}
	ProcessResponse(resp interface{}, root interface{}, models *ytypes.Schema) error
}

// YANGConsumer is a auxillary type to redirect the response
// of an RESTCONF call to a ygot YANG unmarshaler
type YANGConsumer struct {
	Data *bytes.Buffer
}

// Consume reads the received data into a byte buffer
func (yc YANGConsumer) Consume(reader io.Reader, _ interface{}) error {
	_, err := yc.Data.ReadFrom(reader)
	return err
}

// TransportOptions provides an interface for TransportOptions implementations
// for Transports like RESTCONF or gNMI
type TransportOptions interface {
	GetAddress() string
	GetUsername() string
	GetPassword() string
	IsTransportOption()
}
